package org.zheng.demo;

import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertNull;
import static org.junit.jupiter.api.Assertions.assertTrue;

import org.junit.jupiter.api.Test;
import org.zheng.demo.data.Customer;
import org.zheng.demo.data.Order;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

class GsonTest {

	private Customer customer = new Customer("Mary", 5);
	private Order order = new Order(1, null);

	@Test
	void test_default_nonnull() {
		// By default, Gson excludes fields with null values.
		String json = new Gson().toJson(order);

		assertFalse(json.contains("null"), "Gson default excludes null values.");
		System.out.println(json); // {"quantity":1}
		
		// using the GsonBuilder and setting the serializeNulls method.
		String jsonString = new GsonBuilder().create().toJson(customer);

		assertFalse(jsonString.contains("null"), "GsonBuilder default excludes null values.");
		System.out.println(jsonString); // {"id":5,"name":"Mary"}
	}


	@Test
	void test_serialize_null_with_Expose() {
		Gson gson = new GsonBuilder().serializeNulls().excludeFieldsWithoutExposeAnnotation().create();

		customer.setCustTag("NOT_Serialized_field");

		String jsonStr = gson.toJson(customer);
		
		assertFalse(jsonStr.contains("notIncludeInJson"), "not expose field not included");
		assertTrue(jsonStr.contains("null"), "Json string contains null when serializeNulls() is configured");
		System.out.println(jsonStr);
		// {"email":null,"id":5,"name":"Mary"}

		Customer desObj = gson.fromJson(jsonStr, Customer.class);
		assertNull(desObj.getCustTag());

	}

	@Test
	void test_serializeNulls() {
		customer.addOrder(order);

		Gson gson = new GsonBuilder().serializeNulls().create();
		String jsonString = gson.toJson(customer);

		assertTrue(jsonString.contains("custTag"),
				"not calling excludeFieldsWithoutExposeAnnotation, so ignore the @expose.");
		assertTrue(jsonString.contains("null"), "Json string contains null when serializeNulls() is configured");

		System.out.println(jsonString);
		// {"custTag":null,"email":null,"id":5,"name":"Mary","orders":[{"productName":null,"quantity":1}]}

	}
}
